/*
 * Copyright (c) 2013 Emmanuel Dreyfus
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *        This product includes software developed by Emmanuel Dreyfus
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <dirent.h>
#include <signal.h>
#include <getopt.h>
#include <err.h>
#include <errno.h>
#include <sysexits.h>
#include <time.h>
#include <paths.h>
#include <sys/param.h>

#ifndef STATE_OK
#define STATE_OK        0
#define STATE_WARNING   1
#define STATE_CRITICAL  2
#define STATE_UNKNOWN   3
#endif

#define DOT_PID ".pid"

int
main(argc, argv)
	int argc;
	char **argv;
{
	char *dirname = _PATH_VARRUN;
	DIR *dirp;
	struct dirent *dp;
	char running[4096];
	char dead[4096];
	int state = STATE_OK;
	int i, ch;

	while ((ch = getopt(argc, argv, "d:")) != -1) {
		switch (ch) {
		case 'd':
			dirname = optarg;
			break;
		default:
			printf("usage: %s [-d dirname] [service1 ...]\n",
			       getprogname());
			exit(EX_USAGE);
		}
	}

	/* Skip program name */
	if (optind == 0)
		optind = 1;

	argv += optind;
	argc -= optind;

	if (chdir(dirname) != 0) {
		printf("CRITICAL: failed to chdir to %d: %s\n", 
		       dirname, strerror(errno));
		exit(STATE_CRITICAL);
	}
		
	for (i = 0; i < argc; i++) {
		char name[MAXPATHLEN + 1];

		(void)snprintf(name, MAXPATHLEN, "%s%s", argv[i], DOT_PID);
		if (access(name, F_OK) != 0) {
			snprintf(dead, sizeof(dead), "%s%s ",
				 dead, argv[i]);
			state = STATE_CRITICAL;
		}
	}

	if ((dirp = opendir(dirname)) == NULL) {
		printf("CRITICAL: failed to open %s: %s\n", 
		       dirname, strerror(errno));
		exit(STATE_CRITICAL);
	}

	while ((dp = readdir(dirp)) != NULL) {
		char *suffix;
		FILE *fp;
		pid_t pid;
	
		if (dp->d_namlen == 0)
			continue;
		
		if ((suffix = strrchr(dp->d_name, (int)'.')) == NULL)
			continue;

		if (strcmp(suffix, DOT_PID) != 0)
			continue;

		if ((fp = fopen(dp->d_name, "r")) == NULL) {
			printf("UNKNOWN: cannot open %s%s: %s\n",
			       dirname, dp->d_name, strerror(errno));
			exit(STATE_UNKNOWN);
		}

		*suffix = '\0';
			
		if (fscanf(fp, "%d", &pid) != 1) {
			printf("UNKNOWN: unexpected data from %s%s.pid\n",
			       dirname, dp->d_name);
			exit(STATE_UNKNOWN);
		}

		(void)fclose(fp);

		if (kill(pid, 0) != 0) {
			if (errno != ESRCH) {
				printf("UNKNOWN: kill %s [%d] failed: %s\n",
				       dp->d_name, pid, strerror(errno));
				exit(STATE_UNKNOWN);
			}
			
			snprintf(dead, sizeof(dead), "%s%s ",
				 dead, dp->d_name);
			state = STATE_CRITICAL;
		} else {
			snprintf(running, sizeof(dead), "%s%s ",
				 running, dp->d_name);
		}
	}

	(void)closedir(dirp);

	if (state == STATE_OK)
		printf("OK: %s\n", running);
	else
		printf("CRITICAL: %s\n", dead);
	
	return STATE_OK;;
}
